/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.vvc.shared.core.ui.objects.GlobalConfigurationResource");

	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.Constants");
	var Constants = com.ibm.team.vvc.shared.core.ui.objects.Constants;
	
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.GlobalConfigurationConstants");
	var GlobalConfigurationConstants = com.ibm.team.vvc.shared.core.ui.objects.GlobalConfigurationConstants;

	
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.VvcObject");
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.util.SecurityServiceUtils");
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
	var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;
	
	var DC_TERMS = "http://purl.org/dc/terms/";

	dojo.declare("com.ibm.team.vvc.shared.core.ui.objects.GlobalConfigurationResource",
			com.ibm.team.vvc.shared.core.ui.objects.VvcObject, {
		
		getFrontServiceId: function() {
			return "globalConfigurations";
		},

		getTitle: function() {
			return this.getProperty(Constants.TITLE);
		},
		
		setTitle: function(newTitle) {
			this.setProperty(Constants.TITLE, newTitle, "literal");
		},		
		
		getDescription: function() {
			return this.getProperty(Constants.DESCRIPTION);
		},
		
		setDescription: function(newDesc) {
			this.setProperty(Constants.DESCRIPTION, newDesc, "literal");
		},
		
		getMutable: function() {
			//be careful, it is stored as a string!
			return this.getProperty(GlobalConfigurationConstants.GC_MUTABLE) === "true";
		},
		
		setMutable: function(newMutable) {
			this.setBooleanProperty(GlobalConfigurationConstants.GC_MUTABLE, newMutable);
		},		
		
		getComponent: function() {
			return this.getProperty(GlobalConfigurationConstants.GC_COMPONENT);
		},
		
		setComponent: function(newComponent) {
			this.setProperty(GlobalConfigurationConstants.GC_COMPONENT, newComponent, "uri");
		},		
		
		getPreviousBaselines: function() {
			return this.getArrayProperty(GlobalConfigurationConstants.GC_PREVIOUS_BASELINES);
		},
		
		setPreviousBaselines: function(value) {
			this.setPropertyArray(GlobalConfigurationConstants.GC_PREVIOUS_BASELINES, value, "uri");
		},
		
		getContributions: function() {
			return this.getArrayProperty(GlobalConfigurationConstants.GC_CONTRIBUTION);
		},
		
		setContributions: function(value) {
			this.setPropertyArray(GlobalConfigurationConstants.GC_CONTRIBUTION, value, "uri");
		},
		
		//////////////// other properties
		
		getCreated: function() {
			return this.getProperty(DC_TERMS + Constants.CREATED);
		},

		getCreator: function() {
			return this.getProperty(DC_TERMS + Constants.CREATOR);
		},

//		getType: function()
//		{
//			return this.getProperty("http://www.w3.org/1999/02/22-rdf-syntax-ns#type");
//		},
		
		constructor: function(args) {
			if (!args) {
				args = {};
			}
			
			this.setUri(args.uri ? args.uri : "https://tobeassigned");

			this.setProperty("http://www.w3.org/1999/02/22-rdf-syntax-ns#type", GlobalConfigurationConstants.GC, "uri");
			
			//don't do the safeMixin -- do it the legible way since we are using setters anyway
			if (args.title) {
				this.setTitle(args.title);
			} else {
				this.setTitle("");	
			}
			
			if (args.description) {
				this.setDescription(args.description);
			} else {
				this.setDescription("");
			}
			
			if (args.mutable) {
				this.setMutable(args.mutable);
			} else {
				this.setMutable(false);	
			}
			
			if (args.component) {
				this.setComponent(args.component);
			}
			
			if (args.previousBaselines) {
				this.setPreviousBaselines(args.previousBaselines);
			} else {
				this.setPreviousBaselines([]);
			}
			
			if (args.contributions) {
				this.setContributions(args.contributions);
			} else {
				this.setContributions([]);
			}
		},
		
		doCreate: function() {
			var saveData = dojo.toJson(this.data);
			var theHeaders = {
					"Accept" : "application/rdf+json",
					"Content-Type" : "application/rdf+json",
				};
			var args = {
				url: VvcUtils.getVvcUri() + "/" + this.getFrontServiceId(),
				headers: theHeaders,
				sync: false,
				handleAs: "json",
				postData: saveData,
				load: function(response, ioArgs) {
					if (ioArgs) {
						var xhr = ioArgs.xhr;
						if (xhr) {
							return xhr.getResponseHeader("Location");
						}
					}
				}
			};
			return jazz.client.xhrPost(args);
		}
		
		
	});
})();