/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corporation 2014. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
dojo.provide("com.ibm.team.vvc.shared.web.ui.util.ConfigurationUiNode");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojox.html.entities");
dojo.require("dojo/on");

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;

dojo.require("jazz.ui.HoverPopup");
var HoverPopup = jazz.ui.HoverPopup;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcCache");
var VvcCache = com.ibm.team.vvc.shared.web.ui.util.VvcCache;

dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
var VvcConstants = com.ibm.team.vvc.shared.web.ui.VvcConstants;

dojo.require("jazz.ui.ResourceLink");
var ResourceLink = jazz.ui.ResourceLink;

dojo.requireLocalization("com.ibm.team.vvc.shared.web.ui.util", "ConfigurationUiNodeMessages");

dojo.declare("com.ibm.team.vvc.shared.web.ui.util.ConfigurationUiNode", [dijit._Widget], {
	
    messages: dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.util", "ConfigurationUiNodeMessages"),
	
	_uri: null,
	_cache: null,
	_includeLink: null,
	_includeHover: null,
	_isTitle: null,
	_parentDomNode: null,
	_highlightSequences: null,
	
    constructor: function(args, containerNode){
    	if(!args){
    		throw "ConfigurationUiNode missing arguments!"
    	}
    	this._uri = args.uri;

    	this._includeLink = args.includeLink !== undefined ? args.includeLink : true;  	    	
    	this._includeHover = args.includeHover !== undefined ? args.includeHover : true;
    	this._isTitle = args.isTitle !== undefined ? args.isTitle: false;
    	this._highlightCharacters = args.highlightCharacters !== undefined ? args.highlightCharacters : [];
    	
    	this._cache = this._getCache();
    	
    	if(containerNode){
    		this._parentDomNode = containerNode;
    	}
    },
    
    _getCache: function(){
    	return new VvcCache({
			storageKey: VvcConstants.CONFIGURATION_CACHE_KEY,
    	});
    },
    
    postCreate: function(){
    	this.domNode = dojo.create("a", {
    		title: this.messages.loading,
    		href: "#",	// Link to nowhere
    		"class": "configurationUiNode"
    	});
    	
    	if(this._isTitle){
    		dojo.addClass(this.domNode, "titleNode");
    	}
    	
    	this.refresh();
    	
    	if(this._parentDomNode){
    		this._parentDomNode.appendChild(this.domNode);
    	}
    },
        
    refresh: function(){	
    	var loadingNode = this._buildLoadingNode();
    	dojo.empty(this.domNode);
    	this.domNode.appendChild(loadingNode);
    	dojo.setAttr(this.domNode, "disabled", "true");
    	
    	// Clear any existing stop clicks, then set a stop-click while the node loads.
    	this._removeStopClick();
		this._setStopClick();
    	
    	dojo.when(this._getContent(), dojo.hitch(this, function(){
    		if(this._includeLink){
        		dojo.removeAttr(this.domNode, "disabled");
        		this._removeStopClick();
    		}
    	}), dojo.hitch(this, function(error){
    		dojo.removeAttr(this.domNode, "disabled");
    		this._buildErrorNode(error);
    	}));
    },
    /**
	  * Prevents the user from clicking on the DOM node.
	  */
    _setStopClick: function(){
        var setStopClick = dojo.connect(this.domNode, "onclick", dojo.hitch(this, function(event){
    		dojo.stopEvent(event); // Stop the DOM events from firing (page redirect)
    		if(this.onclick){
    			this.onclick(); // Faux click event
    		}
    	}));
    	
    	var removeStopClick = dojo.connect(this, "_removeStopClick", dojo.hitch(this, function(){
			dojo.disconnect(setStopClick);
			dojo.disconnect(removeStopClick);
    	}));
    	return setStopClick;
    },
    
    _removeStopClick: function(){
    	// Stub
    },
        
    _buildLoadingNode: function(){
    	var loadingNode = dojo.create("span");
    	
    	var iconNode = dojo.create("span", {
    		"class": "iconNode"
    	});
    	var icon = dojo.create("img", {
    		src: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/spinner.gif'),
			alt: this.messages.loading
    	});
    	iconNode.appendChild(icon);
    	
    	var titleNode = dojo.create("span", {
    		"class": "titleNode",
    		innerHTML: this.messages.loading
    	});
    	
    	loadingNode.appendChild(iconNode);
    	loadingNode.appendChild(titleNode);
    	
    	return loadingNode;
    },
    
    _buildErrorNode: function(error){
    	dojo.empty(this.domNode);
		dojo.removeAttr(this.domNode, "title");
	
    	var contentNode = dojo.create("span", {
    		"class": "error"
    	});
		
    	var iconNode = dojo.create("span", {
    		"class": "iconNode"
    	});
		
    	var titleNode = dojo.create("span", {
    		"class": "titleNode",
    		innerHTML: this.messages.error
    	});
		
    	contentNode.appendChild(iconNode);
    	contentNode.appendChild(titleNode);
    			
		var onClickHandler = dojo.connect(this.domNode, "onclick", this, function(event){
			dojo.stopEvent(event);
			dojo.disconnect(onClickHandler);
			this.refresh();
		});
		
		var messageNode = dojo.create("div", {
			innerHTML: this.messages.clickHereToRefreshNodeTitle
		});
    	
    	var errorPopup = new HoverPopup({
    		around: contentNode,
    		createContents: dojo.hitch(this, function(){
    			return messageNode;
    		})
    	});
    	
		this.domNode.appendChild(contentNode);
    },
        
    _getContent: function(){
    	var cacheDef = this._cache.get(this._uri); 
    	dojo.when(cacheDef, dojo.hitch(this, function(conf){
    		var contentNode = this._buildContentNode(conf);
    		dojo.empty(this.domNode);
    		dojo.removeAttr(this.domNode, "title");
    		
    		if(conf.archive){
    			dojo.addClass(this.domNode, "archived");
    			this._includeLink = false;
    		}
    		
    		if(this._includeLink){
    			dojo.attr(this.domNode, "href", this.getEditorLink(conf));
    		}
    		this.domNode.appendChild(contentNode);
    	}));
    	return cacheDef;
    },
    
    _buildContentNode: function(conf){
    	var contentNode = dojo.create("span", {
    		"class": conf.type,
    		archived: conf.archive
    	});
    	
    	var iconNode = dojo.create("span", {
    		"class": "iconNode"
    	});
    	
    	var titleNode = dojo.create("span", {
    		"class": "titleNode",
    		innerHTML: this.buildTitleNode(dojox.html.entities.encode(conf.title))
    	});
    	
    	if(conf.archive){
    		this._includeHover = false;
    	}
    	
    	if(this._includeHover){
    		this.getHover(conf, this.domNode);
    	}
    	
    	contentNode.appendChild(iconNode);
    	contentNode.appendChild(titleNode);
    	
    	return contentNode;
    },
    
    getEditorLink: function(conf){
    	var queryParams = {
    		contextUri: conf.uri,
    		spaceUri: conf.spaceUri
    	};
    	
    	if(dojo.config.isDebug){
    		queryParams.debug = true;
    	}
    	
    	var editorAction = conf.type === "workspace" ? VvcConstants.EDIT_WORKSPACE : VvcConstants.EDIT_SNAPSHOT;
    	
    	var actionParams = {
    		action: editorAction
    	};
    	
    	return VvcUtils.getVvcUri() + "/web?" +  dojo.objectToQuery(queryParams) + "#" + dojo.objectToQuery(actionParams);
    },
    
    getHover: function(conf, titleNode){
    	var resourceLink = new ResourceLink({
			uri: conf.uri,
			label: dojox.html.entities.encode(conf.title),
			around: titleNode,
			lazyFetch: true
    	});
    },
    
    buildTitleNode: function(title){
    	if(!title || title.length === 0) {
    		return "";
    	}
    	
    	var newTitle = title;
    	if(this._highlightCharacters.length > 0){
    		var start = title.toLocaleLowerCase().indexOf(this._highlightCharacters.toLocaleLowerCase());
    		var end = start + this._highlightCharacters.length;
    		
    		// start < 0 means the string was not found.
    		if(start >= 0){
    			newTitle = title.substring(0, start);
    			newTitle += "<b>" + title.substring(start, end) + "</b>";
    			newTitle += this.buildTitleNode(title.substring(end));
    		}
    	}
    	return newTitle;
    },
    
});
   
})();
